@@ -28,9 +28,6 @@
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
 
-#include <telepathy-glib/account-manager.h>
-#include <telepathy-glib/util.h>
-
 #include <libempathy/empathy-utils.h>
 
 #include "empathy-ui-utils.h"
@@ -280,7 +277,7 @@ account_manager_prepared_cb (GObject *source_object,
       return;
     }
 
-  accounts = tp_account_manager_get_valid_accounts (manager);
+  accounts = tp_account_manager_dup_valid_accounts (manager);
 
   for (l = accounts; l != NULL; l = l->next)
     {
@@ -301,7 +298,7 @@ account_manager_prepared_cb (GObject *source_object,
           self, 0);
     }
 
-  g_list_free (accounts);
+  g_list_free_full (accounts, g_object_unref);
 
   if (self->priv->select_when_ready != NULL)
     {
