@@ -164,30 +164,6 @@ free_queued_item (QueuedItem *item)
   g_slice_free (QueuedItem, item);
 }
 
-static void
-theme_adium_update_enable_webkit_developer_tools (EmpathyThemeAdium *self)
-{
-  WebKitWebView *web_view = WEBKIT_WEB_VIEW (self);
-  gboolean enable_webkit_developer_tools;
-
-  enable_webkit_developer_tools = g_settings_get_boolean (
-      self->priv->gsettings_chat,
-      EMPATHY_PREFS_CHAT_WEBKIT_DEVELOPER_TOOLS);
-
-  g_object_set (G_OBJECT (webkit_web_view_get_settings (web_view)),
-      "enable-developer-extras", enable_webkit_developer_tools, NULL);
-}
-
-static void
-theme_adium_notify_enable_webkit_developer_tools_cb (GSettings *gsettings,
-    const gchar *key,
-    gpointer user_data)
-{
-  EmpathyThemeAdium *self = user_data;
-
-  theme_adium_update_enable_webkit_developer_tools (self);
-}
-
 static gboolean
 theme_adium_navigation_policy_decision_requested_cb (WebKitWebView *view,
     WebKitWebFrame *web_frame,
@@ -555,7 +531,8 @@ theme_adium_append_html (EmpathyThemeAdium *self,
     const gchar *message_classes,
     gint64 timestamp,
     gboolean is_backlog,
-    gboolean outgoing)
+    gboolean outgoing,
+    PangoDirection direction)
 {
   GString *string;
   const gchar *cur = NULL;
@@ -614,9 +591,22 @@ theme_adium_append_html (EmpathyThemeAdium *self,
         }
       else if (theme_adium_match (&cur, "%messageDirection%"))
         {
-          /* FIXME: The text direction of the message
-           * (either rtl or ltr)
-           */
+          switch (direction)
+            {
+              case PANGO_DIRECTION_LTR:
+              case PANGO_DIRECTION_TTB_LTR:
+              case PANGO_DIRECTION_WEAK_LTR:
+                replace = "ltr";
+                break;
+              case PANGO_DIRECTION_RTL:
+              case PANGO_DIRECTION_TTB_RTL:
+              case PANGO_DIRECTION_WEAK_RTL:
+                replace = "rtl";
+                break;
+              case PANGO_DIRECTION_NEUTRAL:
+              default:
+                break;
+            }
         }
       else if (theme_adium_match (&cur, "%senderDisplayName%"))
         {
@@ -742,11 +732,12 @@ theme_adium_append_html (EmpathyThemeAdium *self,
 
 static void
 theme_adium_append_event_escaped (EmpathyThemeAdium *self,
-    const gchar *escaped)
+    const gchar *escaped,
+    PangoDirection direction)
 {
   theme_adium_append_html (self, "appendMessage",
       self->priv->data->status_html, escaped, NULL, NULL, NULL,
-      NULL, "event", empathy_time_get_current (), FALSE, FALSE);
+      NULL, "event", empathy_time_get_current (), FALSE, FALSE, direction);
 
   /* There is no last contact */
   if (self->priv->last_contact)
@@ -851,6 +842,7 @@ empathy_theme_adium_append_message (EmpathyThemeAdium *self,
   gboolean is_backlog;
   gboolean consecutive;
   gboolean action;
+  PangoDirection direction;
 
   if (self->priv->pages_loading != 0)
     {
@@ -1024,10 +1016,12 @@ empathy_theme_adium_append_message (EmpathyThemeAdium *self,
           self->priv->data->in_content_html;
     }
 
+  direction = pango_find_base_dir (empathy_message_get_body (msg), -1);
+
   theme_adium_append_html (self, func, html, body_escaped,
       avatar_filename, name_escaped, contact_id,
       service_name, message_classes->str,
-      timestamp, is_backlog, empathy_contact_is_user (sender));
+      timestamp, is_backlog, empathy_contact_is_user (sender), direction);
 
   /* Keep the sender of the last displayed message */
   if (self->priv->last_contact)
@@ -1047,6 +1041,7 @@ empathy_theme_adium_append_event (EmpathyThemeAdium *self,
     const gchar *str)
 {
   gchar *str_escaped;
+  PangoDirection direction;
 
   if (self->priv->pages_loading != 0)
     {
@@ -1054,8 +1049,9 @@ empathy_theme_adium_append_event (EmpathyThemeAdium *self,
       return;
     }
 
+  direction = pango_find_base_dir (str, -1);
   str_escaped = g_markup_escape_text (str, -1);
-  theme_adium_append_event_escaped (self, str_escaped);
+  theme_adium_append_event_escaped (self, str_escaped, direction);
   g_free (str_escaped);
 }
 
@@ -1064,7 +1060,10 @@ empathy_theme_adium_append_event_markup (EmpathyThemeAdium *self,
     const gchar *markup_text,
     const gchar *fallback_text)
 {
-  theme_adium_append_event_escaped (self, markup_text);
+  PangoDirection direction;
+
+  direction = pango_find_base_dir (fallback_text, -1);
+  theme_adium_append_event_escaped (self, markup_text, direction);
 }
 
 void
@@ -1353,31 +1352,28 @@ empathy_theme_adium_message_acknowledged (EmpathyThemeAdium *self,
 }
 
 static gboolean
-theme_adium_button_press_event (GtkWidget *widget,
-    GdkEventButton *event)
+theme_adium_context_menu_cb (EmpathyThemeAdium *self,
+    GtkWidget *default_menu,
+    WebKitHitTestResult *hit_test_result,
+    gboolean triggered_with_keyboard,
+    gpointer user_data)
 {
-  if (event->button == 3)
-    {
-      gboolean developer_tools_enabled;
+  GtkWidget *menu;
+  EmpathyWebKitMenuFlags flags = EMPATHY_WEBKIT_MENU_CLEAR;
 
-      g_object_get (
-          G_OBJECT (webkit_web_view_get_settings (WEBKIT_WEB_VIEW (widget))),
-          "enable-developer-extras", &developer_tools_enabled, NULL);
+  if (g_settings_get_boolean (self->priv->gsettings_chat,
+        EMPATHY_PREFS_CHAT_WEBKIT_DEVELOPER_TOOLS))
+    flags |= EMPATHY_WEBKIT_MENU_INSPECT;
 
-      /* We currently have no way to add an inspector menu
-       * item ourselves, so we disable our customized menu
-       * if the developer extras are enabled. */
-      if (!developer_tools_enabled)
-        {
-          empathy_webkit_context_menu_for_event (
-            WEBKIT_WEB_VIEW (widget), event,
-            EMPATHY_WEBKIT_MENU_CLEAR);
-          return TRUE;
-        }
-    }
+  menu = empathy_webkit_create_context_menu (
+    WEBKIT_WEB_VIEW (self), hit_test_result, flags);
 
-  return GTK_WIDGET_CLASS (
-      empathy_theme_adium_parent_class)->button_press_event (widget, event);
+  gtk_widget_show_all (menu);
+
+  gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3,
+      gtk_get_current_event_time ());
+
+  return TRUE;
 }
 
 void
@@ -1631,7 +1627,6 @@ static void
 empathy_theme_adium_class_init (EmpathyThemeAdiumClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
-  GtkWidgetClass* widget_class = GTK_WIDGET_CLASS (klass);
 
   object_class->finalize = theme_adium_finalize;
   object_class->dispose = theme_adium_dispose;
@@ -1639,8 +1634,6 @@ empathy_theme_adium_class_init (EmpathyThemeAdiumClass *klass)
   object_class->get_property = theme_adium_get_property;
   object_class->set_property = theme_adium_set_property;
 
-  widget_class->button_press_event = theme_adium_button_press_event;
-
   g_object_class_install_property (object_class, PROP_ADIUM_DATA,
       g_param_spec_boxed ("adium-data",
         "The theme data",
@@ -1680,17 +1673,12 @@ empathy_theme_adium_init (EmpathyThemeAdium *self)
       G_CALLBACK (theme_adium_load_finished_cb), NULL);
   g_signal_connect (self, "navigation-policy-decision-requested",
         G_CALLBACK (theme_adium_navigation_policy_decision_requested_cb), NULL);
+  g_signal_connect (self, "context-menu",
+      G_CALLBACK (theme_adium_context_menu_cb), NULL);
 
   self->priv->gsettings_chat = g_settings_new (EMPATHY_PREFS_CHAT_SCHEMA);
   self->priv->gsettings_desktop = g_settings_new (
     EMPATHY_PREFS_DESKTOP_INTERFACE_SCHEMA);
-
-  g_signal_connect (self->priv->gsettings_chat,
-    "changed::" EMPATHY_PREFS_CHAT_WEBKIT_DEVELOPER_TOOLS,
-    G_CALLBACK (theme_adium_notify_enable_webkit_developer_tools_cb),
-    self);
-
-  theme_adium_update_enable_webkit_developer_tools (self);
 }
 
 EmpathyThemeAdium *
@@ -1739,13 +1727,8 @@ void
 empathy_theme_adium_show_inspector (EmpathyThemeAdium *self)
 {
   WebKitWebView *web_view = WEBKIT_WEB_VIEW (self);
-  WebKitWebInspector *inspector;
 
-  g_object_set (G_OBJECT (webkit_web_view_get_settings (web_view)),
-      "enable-developer-extras", TRUE, NULL);
-
-  inspector = webkit_web_view_get_inspector (web_view);
-  webkit_web_inspector_show (inspector);
+  empathy_webkit_show_inspector (web_view);
 }
 
 gboolean
